package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.LsAdmin;
import com.taauav.admin.query.LsAdminQuery;
import com.taauav.admin.service.ILsAdminService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 外包人员表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@RestController
@RequestMapping("/lsadmin")
public class LsAdminController extends BaseController {

    @Autowired
    private ILsAdminService lsAdminService;

    @Autowired
    private Response response;

    private static final String controllerName = "lsadmin";

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) LsAdminQuery query) {
        return lsAdminService.getList(query);
    }

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody(required = true) @Valid LsAdmin entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return lsAdminService.add(entity);
    }

    /**
     * 获取用户详情
     *
     * @param id 用户ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return lsAdminService.info(id);
    }

    /**
     * 编辑用户详情
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid LsAdmin entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return lsAdminService.edit(entity);
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response delete(Integer id) {
        return lsAdminService.delete(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody LsAdmin entity) {
        return lsAdminService.setStatus(entity);
    }

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/resetPwd")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.PASSWORD_PERMISSION)
    public Response resetPwd(@RequestBody LsAdmin entity) {
        return lsAdminService.resetPwd(entity);
    }

    /**
     * 基础数据
     *
     * @return
     */
    @GetMapping("/getBaseData")
    public Response getBaseData() {
        return response.success(lsAdminService.getBaseData());
    }

    /**
     * 获取河长列表
     *
     * @return
     */
    @PostMapping("/getDriverManagerList")
    public Response getDriverManagerList() {
        return lsAdminService.getDriverManagerList();
    }

}
