package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.LsAuthGroup;
import com.taauav.admin.query.LsAuthGroupQuery;
import com.taauav.admin.service.ILsAuthGroupService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 外部用户角色表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@RestController
@RequestMapping("/lsauthgroup")
public class LsAuthGroupController extends BaseController {

    @Autowired
    private ILsAuthGroupService lsAuthGroupService;

    @Autowired
    private Response response;

    private static final String controllerName = "lsauthgroup";

    /**
     * 获取角色列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) LsAuthGroupQuery query) {
        return lsAuthGroupService.getList(query);
    }

    /**
     * 添加角色
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody(required = true) LsAuthGroup entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return lsAuthGroupService.add(entity);
    }

    /**
     * 获取角色详情
     *
     * @param id 角色ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return lsAuthGroupService.info(id);
    }

    /**
     * 编辑角色
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody(required = false) LsAuthGroup entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return lsAuthGroupService.edit(entity);
    }

    /**
     * 删除角色
     *
     * @param id 角色ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response delete(Integer id) {
        return lsAuthGroupService.delete(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody(required = true) LsAuthGroup entity) {
        return lsAuthGroupService.setStatus(entity);
    }

    /**
     * 获取角色列表
     *
     * @return
     */
    @PostMapping("/getAuthGroupList")
    public Response getAuthGroupList() {
        return lsAuthGroupService.getAuthGroupList();
    }

}
