package com.taauav.admin.controller;

import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.service.IStatisticsMapService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigInteger;

/**
 * <p>地图统计数据</p>
 *
 * @author : dyg
 * @date : 2019-12-07 14:00
 **/
@RequestMapping("statisticsMap")
@RestController
public class StatisticsMapController {
    @Autowired
    private IStatisticsMapService iStatisticsMapService;

    /**
     * 获取所有区域数据
     * @return
     */
    @GetMapping("index")
    @RequiresPermissions("statisticsMap:"+ PermissionConstants.LIST_PERMISSION)
    public Response index() {
        return iStatisticsMapService.getIndexData();
    }

    /**
     * 根据区域编号获取所有河道数据
     * @param driverArea
     * @return
     */
    @GetMapping("driverArea")
    @RequiresPermissions("statisticsMap:driverArea")
    public Response area(BigInteger driverArea, Integer type) {
        return iStatisticsMapService.getDriversByDriverArea(driverArea, type);
    }

    /**
     * 根据河道编号获取图片数据
     * @param inspectDriverId
     * @return
     */
    @GetMapping("fileList")
    @RequiresPermissions("statisticsMap:fileList")
    public Response fileList(Integer inspectDriverId, Integer type) {
        return iStatisticsMapService.getFileListByInspectDriverId(inspectDriverId, type);
    }
}
