package com.taauav.admin.entity;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.*;

/**
 * <p>
 * 河道管理表
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("tauv_driver")
public class TauvDriver extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 河道编码
     */
    private String driverSn;

    /**
     * 河道名称
     */
    @NotEmpty(message = "河道名称不能为空")
    @Length(min = 2, max = 20, message = "河道名称长度为2-20个字")
    private String name;

    /**
     * 河道类型：1河流 2水库 3湖泊 4重点塘坝
     */
    @NotNull(message = "请选择河道类型")
    private Integer type;

    /**
     * 河道层级:3县/区 4镇 5村
     */
    private Integer level;

    /**
     * 河长
     */
    @NotNull(message = "河长不能为空")
    private Integer driverManager;

    /**
     * 河道长度(单位：km)
     */
    @NotEmpty(message = "长度不能为空")
    private String length;

    /**
     * 起点
     */
    @Length(max = 20, message = "起点长度不能超过20个字")
    private String startPoint;

    /**
     * 终点
     */
    @Length(max = 20, message = "终点长度不能超过20个字")
    private String endPoint;

    /**
     * 河道编码
     */
    private BigInteger driverArea;

    /**
     * 备注
     */
    @Length(max = 255, message = "备注最大长度为255")
    private String note;

    /**
     * 河流类型
     *
     * @return
     */
    public Map<Integer, String> getTypeList() {
        Map<Integer, String> map = new HashMap<>(4);
        map.put(1, "河流");
        map.put(2, "水库");
        map.put(3, "湖泊");
        map.put(4, "重点塘坝");
        return map;
    }

    /**
     * 状态
     *
     * @return
     */
    public Map<Integer, String> getStatusList() {
        Map<Integer, String> map = new HashMap<>(2);
        map.put(1, "正常");
        map.put(2, "停用");
        return map;
    }

    /**
     * 河湖层级
     *
     * @return
     */
    public Map<Integer, String> getLevelList() {
        Map<Integer, String> map = new HashMap<>(3);
        map.put(3, "县/区");
        map.put(4, "镇");
        map.put(5, "村");
        return map;
    }
}
