package com.taauav.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.common.bean.Menu;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysAuthGroup;
import com.taauav.admin.entity.SysAuthRule;
import com.taauav.admin.mapper.SysAuthRuleMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysAuthGroupService;
import com.taauav.admin.service.ISysAuthRuleService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.MenuTree;
import com.taauav.common.util.ShiroUtils;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.util.*;

/**
 * <p>
 * 权限规则表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@Service
@Slf4j
@Data
public class SysAuthRuleServiceImpl extends BaseServiceImpl<SysAuthRuleMapper, SysAuthRule> implements ISysAuthRuleService {
    @Value("${spring.pageSize}")
    private Integer pageSize;

    @Resource
    private SysAuthRuleMapper authRuleMapper;
    @Autowired
    private Response response;
    @Resource
    private Validator validator;

    @Autowired
    private ISysAuthGroupService iSysAuthGroupService;
    @Autowired
    private ISysAdminService iSysAdminService;

    /**
     * 获取用户拥有的菜单权限
     *
     * @return
     */
    @Override
    public List<SysAuthRule> getAuthRules() {
        Integer adminId = ShiroUtils.getAdminId();
        SysAdmin admin = iSysAdminService.getAdminInfo(adminId);
        String rules = admin.getRules();
        QueryWrapper<SysAuthRule> queryWrapper = new QueryWrapper<>();
        //超级管理员拥有所有权限
        if (adminId.equals(1)) {
            queryWrapper.eq("mark", 1);
        } else {
            List<SysAuthGroup> groupList = iSysAuthGroupService.getAuthGroups(adminId);
            if (groupList != null) {
                if (rules.length() > 0) {
                    rules += ",";
                }
                for (SysAuthGroup group : groupList) {
                    rules += group.getRules() + ",";
                }
            }
            rules = rules.substring(0, rules.length() - 1);
            Object[] ruleList = rules.split(",");
            queryWrapper.in("id", ruleList);
            queryWrapper.eq("mark", 1);
            queryWrapper.eq("status", 1);
        }

        List<SysAuthRule> authRuleList = authRuleMapper.selectList(queryWrapper);
        return authRuleList;
    }

    /**
     * 根据编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public SysAuthRule getInfoById(Integer id) {
        SysAuthRule info = authRuleMapper.selectById(id);
        return info;
    }

    /**
     * 获取所有菜单-列表页
     *
     * @param request
     * @return
     */
    @Override
    public Response getRulePageList(Map<String, String> request) {
        Map<String, Object> query = new HashMap<>();
        query.put("pid", "0");
        query.put("field", "id,`name`,permission,status,pid,sort,title,icon,create_user,has_child,type");
        List<Map<String, Object>> list = getListByPid(query);
        if (list != null) {
            SysAuthRule authRule = new SysAuthRule();
            Map<Integer, String> statusList = authRule.getStatusList();
            for (Map<String, Object> auth : list) {
                query.put("pid", auth.get("id"));
                List<Map<String, Object>> child = getListByPid(query);
                if (child != null) {
                    for (Map<String, Object> auth2 : child) {
                        query.put("pid", auth2.get("id"));
                        List<Map<String, Object>> childList = getListByPid(query);
                        auth2.put("children", childList);
                    }
                }
                auth.put("children", child);
                auth.put("statusText", statusList.get(auth.get("status")));
            }
        }
        return response.success(list);
    }

    /**
     * 菜单页面-编辑
     *
     * @param authRule
     * @return
     */
    @Override
    public Response editRule(SysAuthRule authRule) {
        log.info(JSON.toJSONString(authRule));
        Set<ConstraintViolation<SysAuthRule>> violations = validator.validate(authRule);
        for (ConstraintViolation<SysAuthRule> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        Integer id = authRule.getId();
        if (StringUtils.isEmpty(id)) {
            return response.failure("数据不存在");
        }
        SysAuthRule info = authRuleMapper.selectById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        if (!(info.getName().equals(authRule.getName()))) {
            Integer num = getNumByType(authRule, "name");
            if (num > 0) {
                return response.failure("地址已经存在");
            }
        }
        if (!(info.getPermission().equals(authRule.getPermission()))) {
            Integer num = getNumByType(authRule, "permission");
            if (num > 0) {
                return response.failure("权限地址已经存在");
            }
        }
        if (!(info.getTitle().equals(authRule.getTitle()))) {
            Integer num = getNumByType(authRule, "title");
            if (num > 0) {
                return response.failure("名称已经存在");
            }
        }
        boolean result = this.editData(authRule);
        if (!result) {
            return response.failure("修改失败");
        }
        return response.success("修改成功");
    }

    /**
     * 菜单页面-添加
     *
     * @param authRule
     * @return
     */
    @Override
    public Response addRule(SysAuthRule authRule) {
        Set<ConstraintViolation<SysAuthRule>> violations = validator.validate(authRule);
        for (ConstraintViolation<SysAuthRule> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        //校验名称是否已经存在
        Integer num = getNumByType(authRule, "name");
        if (num > 0) {
            return response.failure("地址已经存在");
        }
//        num= getNumByType(authRule,"permission");
//        if (num > 0) {
//            return response.failure("权限地址已经存在");
//        }
        num = getNumByType(authRule, "title");
        if (num > 0) {
            return response.failure("名称已经存在");
        }
        Integer pid = authRule.getPid();
        if (!StringUtils.isEmpty(pid)) {
            SysAuthRule authRuleInfo = authRuleMapper.selectById(pid);
            if (!StringUtils.isEmpty(authRuleInfo)) {
                Integer level = authRuleInfo.getLevel();
                authRule.setLevel(level + 1);
            }
        }
        boolean result = this.addData(authRule);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 菜单页面-删除
     *
     * @param authRule
     * @return
     */
    @Override
    public Response delRule(SysAuthRule authRule) {
        Integer id = authRule.getId();
        if (StringUtils.isEmpty(id)) {
            return response.failure("数据不存在");
        }
        SysAuthRule info = authRuleMapper.selectById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        SysAuthRule delSet = new SysAuthRule();
        delSet.setMark(0);
        delSet.setId(authRule.getId());
        authRuleMapper.updateById(delSet);
        return response.success("删除成功");
    }

    /**
     * 获取用户拥有的菜单权限
     *
     * @return
     */
    @Override
    public List<Map<String, Object>> getRuleList(Integer adminId) {
        List<Map<String, Object>> list;
        SysAdmin admin = ShiroUtils.getAdminInfo();
        Map<String, Object> query = new HashMap<>();
        if (adminId.equals(1)) {
            query.put("pid", "0");
            //1 显示
            query.put("status", "1");
            //2 是菜单
            query.put("type", "2");
            //获取一级
            List<Map<String, Object>> pList = getListByPid(query);
            list = getDataList(pList, query);
        } else {
            //根据个人的权限获取数据
            String rules = admin.getRules();
            List<SysAuthGroup> groupList = iSysAuthGroupService.getAuthGroups(adminId);
            if (groupList != null) {
                for (SysAuthGroup group : groupList) {
                    rules += group.getRules() + ",";
                }
            }
            rules = rules.substring(0, rules.length() - 1);
            String[] ruleList = rules.split(",");
            query.put("pid", "0");
            //1 显示
            query.put("status", "1");
            //2 是菜单
            query.put("type", "2");
            query.put("ruleIds", ruleList);
            //获取一级
            List<Map<String, Object>> pList = getListByPid(query);
            list = getDataList(pList, query);
        }
        return list;
    }

    /**
     * 修改菜单状态
     *
     * @param map
     * @return
     */
    @Override
    public Response editStatus(Map<String, String> map) {
        String idStr = map.get("id");
        String status = map.get("status");
        if (StringUtils.isEmpty(idStr)) {
            return response.failure("数据不存在");
        }
        if ("2".equals(status) || "1".equals(status)) {
            SysAuthRule sysAuthRule = new SysAuthRule();
            sysAuthRule.setId(Integer.valueOf(idStr));
            sysAuthRule.setStatus(Integer.valueOf(status));
            authRuleMapper.updateById(sysAuthRule);
            return response.success("操作成功");
        } else {
            return response.failure("状态不合法");
        }
    }

    /**
     * 根据父级编号获取数据
     *
     * @param pid
     * @return
     */
    @Override
    public List<Map<String, Object>> getListByPid(Integer pid) {
        QueryWrapper queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mark", 1);
        queryWrapper.eq("pid", pid);
        queryWrapper.select("id,title");
        List<Map<String, Object>> list = authRuleMapper.selectMaps(queryWrapper);
        return list;
    }

    @Override
    public List<Menu> buildTreeMenu() {
        List<Menu> menuList = new ArrayList<>();
        QueryWrapper<SysAuthRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("id");
        queryWrapper.orderByAsc("sort");
        List<SysAuthRule> list = authRuleMapper.selectList(queryWrapper);
        if (list != null) {
            for (SysAuthRule sysAuthRule : list) {
                Menu menu = new Menu(sysAuthRule.getId().toString(), sysAuthRule.getName(), sysAuthRule.getTitle(),
                        sysAuthRule.getPid().toString(), "Y");
                menuList.add(menu);
            }
            log.info(JSON.toJSONString(menuList));
            MenuTree menuTree = new MenuTree(menuList);
            log.info(JSON.toJSONString(menuTree));
            menuList = menuTree.buildTree();
        }
        return menuList;
    }

    /**
     * 根据一级菜单 获取子菜单数据
     *
     * @param pList
     * @param query
     * @return
     */
    private List<Map<String, Object>> getDataList(List<Map<String, Object>> pList, Map<String, Object> query) {
        if (pList != null) {
            for (Map<String, Object> auth : pList) {
                //获取二级
                query.put("pid", auth.get("id"));
                List<Map<String, Object>> child = getListByPid(query);
                if (child != null) {
                    for (Map<String, Object> authRule : child) {
                        //获取三级
                        query.put("pid", authRule.get("id"));
                        query.put("type", "1");
                        List<Map<String, Object>> childlist = getListByPid(query);
                        if (childlist != null) {
                            authRule.put("children", childlist);
                        }
                    }
                    auth.put("children", child);
                }
            }
        }
        return pList;
    }

    /**
     * 根据条件查询数据
     *
     * @param map
     * @return
     */
    private List<Map<String, Object>> getListByPid(Map<String, Object> map) {
        QueryWrapper queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mark", 1);
        String pid = "pid";
        String status = "status";
        String type = "type";
        String rules = "ruleIds";
        if (map.containsKey(pid)) {
            queryWrapper.eq("pid", Integer.valueOf(map.get("pid").toString()));
        }
        if (map.containsKey(status)) {
            queryWrapper.eq("status", Integer.valueOf(map.get("status").toString()));
        }
        if (map.containsKey(type)) {
            queryWrapper.eq("type", Integer.valueOf(map.get("type").toString()));
        }
        if (map.containsKey(rules)) {
            queryWrapper.in("id", map.get("ruleIds"));
        }
        if (map.containsKey("field")) {
            queryWrapper.select(map.get("field").toString());
        }
        queryWrapper.orderByAsc("sort");
        queryWrapper.orderByDesc("id");
        List<Map<String, Object>> list = authRuleMapper.selectMaps(queryWrapper);
        if (list != null) {
            SysAuthRule authRule = new SysAuthRule();
            Map<Integer, String> statusList = authRule.getStatusList();
            for (Map<String, Object> val : list) {
                Integer createUser = Integer.valueOf(val.get("create_user").toString());
                SysAdmin info = iSysAdminService.findById(createUser);
                if (info != null) {
                    val.put("format_create_user", info.getRealname());
                } else {
                    val.put("format_create_user", "");
                }
                val.put("statusText", statusList.get(val.get("status")));
            }
        }
        return list;
    }

    /**
     * 根据条件获取数据
     *
     * @param authRule
     * @param str
     * @return
     */
    private Integer getNumByType(SysAuthRule authRule, String str) {
        //校验名称是否已经存在
        QueryWrapper query = new QueryWrapper<>();
        String name = "name";
        String permission = "permission";
        String title = "title";
        if (name.equals(str)) {
            query.eq("name", authRule.getName());
        } else if (permission.equals(str)) {
            query.eq("permission", authRule.getPermission());
        } else if (title.equals(str)) {
            query.eq("title", authRule.getTitle());
        }
        query.eq("mark", 1);
        Integer num = authRuleMapper.selectCount(query);
        return num;
    }

    /**
     * 获取用户权限一维数组
     *
     * @param admin
     * @return
     */
    @Override
    public String getPermission(SysAdmin admin) {
        if (admin == null) {
            return null;
        }
        String adminStr = "admin";
        String ruleIds = "";
        if (adminStr.equals(admin.getUsername())) {
            ruleIds = "*";
        } else {
            String rules = admin.getRules();
            List<SysAuthGroup> groupList = iSysAuthGroupService.getAuthGroups(admin.getId());
            if (groupList != null) {
                if (rules.length() > 0) {
                    rules += ",";
                }
                for (SysAuthGroup group : groupList) {
                    rules += group.getRules() + ",";
                }
            }
            rules = rules.substring(0, rules.length() - 1);
            String[] ruleList = rules.split(",");
            Set<String> set = new HashSet<>();
            for (String rule : ruleList) {
                set.add(rule);
            }
            for (String rule : set) {
                ruleIds += rule + ",";
            }
            if (ruleIds.length() > 0) {
                ruleIds = ruleIds.substring(0, ruleIds.length() - 1);
            }
        }
        return ruleIds;
    }
}
